/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright(c) 2015 Nuvoton Technology Corp. All rights reserved.                                         */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/

//***********************************************************************************************************
//  Nuvoton Technology Corp.
//  E-mail: MicroC-8bit@nuvoton.com
//***********************************************************************************************************
//  Application: I2C Function(Slave)
//  SCL => P1.2 ; SDA => P1.3
//
//  Output : I2C status => P0
// **********************************************************************************************************

#include <stdio.h>
#include "N79E715.h"
#include "Typedef.h"
#include "Define.h"
#include "Common.h"
#include "Delay.h"
#include "Version.h"

#define Slave_Address   0xA4

uint8_t u8Data_Received[34], u8Data_Num = 0;

//-----------------------------------------------------------------------------------------------------------
void i2c_isr(void) interrupt 6
{
    switch (I2STA)
    {
    case 0x00:
        STO = 1;
        break;

    case 0x60:
        AA = 1;
        P0 = 0x60;
        break;

    case 0x80:
        P0 = 0x80;
        u8Data_Received[u8Data_Num] = I2DAT;
        u8Data_Num++;

        if (u8Data_Num == 34)
            AA = 0;
        else
            AA = 1;
        break;

    case 0x88:
        P0 = 0x88;
        u8Data_Received[u8Data_Num] = I2DAT;
        u8Data_Num = 0;
        AA = 1;
        break;

    case 0xA0:
        P0 = 0xA0;
        AA = 1;
        break;

    case 0xA8:
        P0 = 0xA0;
        I2DAT = u8Data_Received[u8Data_Num];
        u8Data_Num++;
        AA = 1;
        break;

    case 0xB8:
        P0 = 0xB8;
        I2DAT = u8Data_Received[u8Data_Num];
        u8Data_Num++;
        AA = 1;
        break;

    case 0xC0:
        P0 = 0xC0;
        AA = 1;
        break;

    case 0xC8:
        P0 = 0xC8;
        AA = 1;
        break;
    }

    SI = 0;
    while(STO);
}
//-----------------------------------------------------------------------------------------------------------
void Init_I2C(void)
{
    SDA = 1;                                    // Set SDA and SCL pins high
    SCL = 1;
    P3M1 |= SET_BIT5;                           // Enable P1 Schmitt trigger input

    EI2C = 1;                                   // Enable I2C interrupt by setting EIE bit 0
    EA = 1;

    I2ADDR = 0xA4;                              // Define own slave address
    I2CEN = 1;                                  // Enable I2C circuit
    AA = 1;
}
//-----------------------------------------------------------------------------------------------------------
void main(void)
{
    InitialUART0_Timer1(9600);                  // 9600 Baud Rate @ 11.0592MHz
    Show_Version_Number_To_PC();
    printf ("\n*==========================================================================");
    printf ("\n*  N79E715 Series I2C Slave Interrupt Sample Code. (N79E715_M <--> N79E715_S)");
    printf ("\n*==========================================================================\n");
    Init_I2C();                                 // Initialize I2C function
    printf ("\nI2C Slave Start Receive...\n");
    while(1);
}
//-----------------------------------------------------------------------------------------------------------




