/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright(c) 2015 Nuvoton Technology Corp. All rights reserved.                                         */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/

//***********************************************************************************************************
//  Nuvoton Technology Corp.
//  E-mail: MicroC-8bit@nuvoton.com
//***********************************************************************************************************
//  Application: KBI Function
//  P00~P07 as KBI Input.
//
//  Output : P1.4 toggle when KBI is triggered once
//           P2.1 toggle when KBI is triggered once
//           Count will add "1" when KBI is triggered once
//           UART show count on hyper-terminal
//***********************************************************************************************************

//------------------------- <<< Use Configuration Wizard in Context Menu >>> --------------------------------
//    <h> KBI  Enable
//                  <q0.0> KBI.0
//                  <q0.1> KBI.1
//                  <q0.2> KBI.2
//                  <q0.3> KBI.3
//                  <q0.4> KBI.4
//                  <q0.5> KBI.5
//                  <q0.6> KBI.6
//                  <q0.7> KBI.7
//    </h>
// <h> KBI Low/High Level Trigger Select
//     <o1.0..1> P0 trigger level select
//         <0=> Falling level            <1=> Rising level
//         <2=> Falling/Rising level     <3=> low level
//  </h>
// <o2.0..1> Power Saving Mode Select
//     <1=> Idle mode
//     <2=> Power Down mode
//     <o3.6> UART pin Select
//          <0=> Select P1.0, P1.1 as UART pin(default)
//          <1=> Select P2.6, P2.7 as UART pin(28 pin only)
//-------------------------------- <<< end of configuration section >>> -------------------------------------

#include "N79E715.h"
#include "Typedef.h"
#include "Define.h"
#include "Common.h"
#include "Delay.h"
#include "Version.h"
#include <intrins.h>
#include <stdio.h>
#include <string.h>

#define KBI_Enable          0xFF
#define KBI_Level           0x00
#define POWER_SAVING_MODE   0x02
#define Uart_Port_Sel       0x00

UINT16 u16Count = 0;

//-----------------------------------------------------------------------------------------------------------
void main()
{
    AUXR1 |= Uart_Port_Sel;             // Select P10/P11 as UART pin(default)
    InitialUART0_Timer1(9600);          // 9600 Baud Rate @ 11.0592MHz
    printf ("\n*===================================================================");
    printf ("\n*  Name: N79E715 Series KBI Sample Code.");
    printf ("\n*===================================================================");
    printf ("\nP00~P07 as KBI Input.");
    printf ("\nKBI Demo Start.\n");

    /*********************** Falling edge ***********************/
#if KBI_Level == 0
    P0 = 0xFF;
    KBLS1 = 0x00;                       // Set P0.0~P0.7 Falling Trigger
    KBLS0 = 0x00;
#endif
    /*********************** Rising edge ***********************/
#if KBI_Level == 1
    P0 = 0x00;
    KBLS1 = 0x00;                       // Set P0.0~P0.7 Rising Trigger
    KBLS0 = 0xFF;
#endif
    /******************** Falling/Rising edge ******************/
#if KBI_Level == 2
    P0 = 0xFF;
    KBLS1 = 0xFF;                       // Set P0.0~P0.7 Falling/Rising Trigger
    KBLS0 = 0x00;
#endif
    /************************ Low level ************************/
#if KBI_Level == 3
    P0 = 0xFF;
    KBLS1 = 0xFF;                       // Set P0.0~P0.7 Low level Trigger
    KBLS0 = 0xFF;
#endif

    KBIF = 0x00;                        // Clear KBF
    EKB = 1;                            // Enable KBI Interrupt
    KBIE = KBI_Enable;                  // KBI Enable
    EA  = 1;                            // Enable EA

    while(1)
    {
        PCON |= POWER_SAVING_MODE;      // Enter power down mode or idle mode selected by wizard
        printf ("\nKBI OK! Count = %d\n",u16Count);
        P14 = ~P14;
        P21 = ~P21;
    }

}
//-----------------------------------------------------------------------------------------------------------
void KBI_ISR(void) interrupt 7          // Vector @  0x3B
{
    u16Count++;
    Delay1ms(800);
    KBIF = 0x00;                        // Clear KBF flag
}
//-----------------------------------------------------------------------------------------------------------