/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright(c) 2015 Nuvoton Technology Corp. All rights reserved.                                         */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/

//***********************************************************************************************************
//  Nuvoton Technology Corp.
//  E-mail: MicroC-8bit@nuvoton.com
//***********************************************************************************************************
//  Application: SPI Function
//  Slave receive 0x90 and return 0x4E
//  Slave receive 0x01 and return 0x55
//  Slave receive 0x02 and return 0x56
//  Slave receive 0x03 and return 0x4F
//  Slave receive 0x04 and return 0x54
//
//  Slave send 0x4F and 0x4E to Master after receiving
//
//  Output : P1.4 & P2.1 flash when SPI pass.
//           P0.7 flash when SPI error
//***********************************************************************************************************

//------------------------- <<< Use Configuration Wizard in Context Menu >>> --------------------------------
//     <o0.6> UART pin
//          <0=> Select P1.0, P1.1 as UART pin(default)
//          <1=> Select P2.6, P2.7 as UART pin(28 pin only) 
//     <o1.7> SPI pin Select
//          <0=> Select P1.6, P1.7, P1.4, P0.0 as SPI pin(default)
//          <1=> Select P2.2, P2.3, P2.4, P2.5 as SPI pin(28 pin only)
//-------------------------------- <<< end of configuration section >>> -------------------------------------

#define Uart_Port_Sel   0x00
#define SPI_Pin_Sel     0x80

#include "N79E715.h"
#include <stdio.h>
#include "Typedef.h"
#include "Define.h"
#include "Common.h"
#include "Delay.h"
#include "SPI.h"
#include "Version.h"

UINT8   u8Receive_Data[5];
UINT16  u16CNT = 0;
BIT     SPI_Send_Flag = 0;

//-----------------------------------------------------------------------------------------------------------
void SPI_Error(void)
{
    while(1)                                    // SPI error and P0.7 flash
    {
        P07 = 1;
        Delay1ms(500);
        P07 = 0;
        Delay1ms(500);
    }
}
//-----------------------------------------------------------------------------------------------------------
void Enable_SPI_Interrupt(void)
{
    ESPI = 1;                                   // Enable SPI interrupt
    EA = 1;
}
//-----------------------------------------------------------------------------------------------------------
void SPI_Initial(void)
{
#if (SPI_Pin_Sel == 0x00)
    P3M1 |= SET_BIT4;                           // Enables Schmitt trigger inputs on Port 0.
    P3M1 |= SET_BIT5;                           // Enables Schmitt trigger inputs on Port 1.
#else
    P3M1 |= SET_BIT6;                           // Enables Schmitt trigger inputs on Port 2.
#endif

    clr_MSTR;                                   // SPI in Slave mode
    clr_LSBFE;                                  // MSB first

    clr_CPOL;                                   // The SPI clock is low in idle mode
    set_CPHA;                                   // The data is sample on the second edge of SPI clock

    Enable_SPI_Interrupt();                     // Enable SPI interrupt
    set_SPIEN;                                  // Enable SPI function

    clr_SPIF;                                   // Clear SPI flag
}
//-----------------------------------------------------------------------------------------------------------
void Slave_Receive_Data(void)
{
    SPDR = 0x4E;
    PCON |= SET_BIT0;                           // Enter idle mode
    if(u8Receive_Data[0] != 0x90)               // Receive 1st DATA from master
        SPI_Error();

    SPDR = 0x55;
    PCON |= SET_BIT0;                           // Enter idle mode
    if(u8Receive_Data[1] != 0x01)               // Receive 2nd DATA from master
        SPI_Error();

    SPDR = 0x56;
    PCON |= SET_BIT0;                           // Enter idle mode
    if(u8Receive_Data[2] != 0x02)               // Receive 3rd DATA from master
        SPI_Error();

    SPDR = 0x4F;
    PCON |= SET_BIT0;                           // Enter idle mode
    if(u8Receive_Data[3] != 0x03)               // Receive 4th DATA from master
        SPI_Error();

    SPDR = 0x54;
    PCON |= SET_BIT0;                           // Enter idle mode
    if(u8Receive_Data[4] != 0x04)               // Receive 5th DATA from master
        SPI_Error();
}
//-----------------------------------------------------------------------------------------------------------
void Slave_Transmit_Data(void)
{
    SPI_Send_Flag = 1;
    SPDR = 0x4F;                                // Send 1st data (04F) to Master
    PCON |= SET_BIT0;                           // Enter idle mode

    SPI_Send_Flag = 1;
    SPDR = 0x4E;                                // Send 2nd data (0x4E) to Master
    PCON |= SET_BIT0;                           // Enter idle mode
}
//-----------------------------------------------------------------------------------------------------------
void main(void)
{
    AUXR1 |= SPI_Pin_Sel;                       // Select P1.6, P1.7, P1.4, P0.0 as SPI pin(default)
    AUXR1 |= Uart_Port_Sel;                     // Select P10/P11 as UART pin(default)
    InitialUART0_Timer1(9600);                  // 9600 Baud Rate @ 11.0592MHz 
    Show_Version_Number_To_PC();    
    printf ("\n*===================================================================");
    printf ("\n N79E715 Series SPI Slave(Interrupt) Sample Code.");
    printf ("\n*===================================================================");
    SPI_Initial();

    printf ("\nSPI Start Receive...\n");

    Slave_Receive_Data();                       // Slave receive data from master
    Slave_Transmit_Data();                      // Slave transmit data to master

    ESPI = 0;
    SPDR = 0;
    printf ("\nSPI Test OK!\n");
    while(1)                                    // SPI transmission finish
    {
        P14 = 1;
        P21 = 1;
        Delay1ms(500);
        P14 = 0;
        P21 = 0;
        Delay1ms(500);
    }
}
//-----------------------------------------------------------------------------------------------------------
void SPI_ISR(void) interrupt 9                  // Vector @  0x4B
{
    clr_SPIF;
    if(!SPI_Send_Flag)
    {
        u8Receive_Data[u16CNT] = SPDR;
        u16CNT ++;
    }
    SPI_Send_Flag = 0;
}
//-----------------------------------------------------------------------------------------------------------