/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright(c) 2015 Nuvoton Technology Corp. All rights reserved.                                         */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/

//***********************************************************************************************************
//  Nuvoton Technology Corp.
//  E-mail: MicroC-8bit@nuvoton.com
//***********************************************************************************************************
//  Application: SPI Function
//  Slave receive 0x90 and return 0x4E
//  Slave receive 0x01 and return 0x55
//  Slave receive 0x02 and return 0x56
//  Slave receive 0x03 and return 0x4F
//  Slave receive 0x04 and return 0x54
//
//  Slave send 0x4F and 0x4E to Master after receiving
//
//  Output : P1.4 & P2.1 flash when SPI pass.
//           P0.7 flash when SPI error
//***********************************************************************************************************

//------------------------- <<< Use Configuration Wizard in Context Menu >>> --------------------------------
//     <o0.6> UART pin
//          <0=> Select P1.0, P1.1 as UART pin(default)
//          <1=> Select P2.6, P2.7 as UART pin(28 pin only) 
//     <o1.7> SPI pin Select
//          <0=> Select P1.6, P1.7, P1.4, P0.0 as SPI pin(default)
//          <1=> Select P2.2, P2.3, P2.4, P2.5 as SPI pin(28 pin only)
//-------------------------------- <<< end of configuration section >>> -------------------------------------

#define Uart_Port_Sel   0x00
#define SPI_Pin_Sel     0x80

#include <stdio.h>
#include <intrins.h>
#include <string.h>
#include "N79E715.h"
#include "Typedef.h"
#include "Define.h"
#include "Common.h"
#include "Delay.h"
#include "SPI.h"
#include "Version.h"

//-----------------------------------------------------------------------------------------------------------
void SPI_Error(void)
{
    while(1)                                    // SPI error and P0.7 flash/
    {
        P07 = 1;
        Delay1ms(500);
        P07 = 0;
        Delay1ms(500);
    }
}
//-----------------------------------------------------------------------------------------------------------
void SPI_Initial(void)
{
#if (SPI_Pin_Sel == 0x00)
    P3M1 |= SET_BIT4;                           // Enables Schmitt trigger inputs on Port 0.
    P3M1 |= SET_BIT5;                           // Enables Schmitt trigger inputs on Port 1.
#else
    P3M1 |= SET_BIT6;                           // Enables Schmitt trigger inputs on Port 2.
#endif
    clr_MSTR;                                   // SPI in Slave mode
    clr_LSBFE;                                  // MSB first

    clr_CPOL;                                   // The SPI clock is low in idle mode
    set_CPHA;                                   // The data is sample on the second edge of SPI clock

    set_SPIEN;                                  // Enable SPI function
    clr_SPIF;
}
//-----------------------------------------------------------------------------------------------------------
void Slave_Receive_Data(void)
{
    SPDR = 0x4E;                                // Receive Master 1st DATA
    while(!(SPSR & SET_BIT7));
    clr_SPIF;
    if(SPDR != 0x90)
       SPI_Error();

    SPDR = 0x55;                                // Receive Master 2nd DATA
    while(!(SPSR & SET_BIT7));
    clr_SPIF;
    if(SPDR != 0x01)
       SPI_Error();

    SPDR = 0x56;                                // Receive Master 3rd DATA
    while(!(SPSR & SET_BIT7));
    clr_SPIF;
    if(SPDR != 0x02)
        SPI_Error();

    SPDR = 0x4F;                                // Receive Master 4th DATA
    while(!(SPSR & SET_BIT7));
    clr_SPIF;
    if(SPDR != 0x03)
        SPI_Error();

    SPDR = 0x54;                                // Receive Master 5th DATA
    while(!(SPSR & SET_BIT7));
    clr_SPIF;
    if(SPDR != 0x04)
        SPI_Error();
}
//-----------------------------------------------------------------------------------------------------------
void Slave_tranmit_Data(void)
{
    SPDR = 0x4F;                                // Send 1st data (0x4F) to Master
    while(!(SPSR & SET_BIT7));
    clr_SPIF;
    if(SPDR != 0xFF)
        SPI_Error();

    SPDR = 0x4E;                                // Send 2nd data (0x4E) to Master
    while(!(SPSR & SET_BIT7));
    clr_SPIF;
    if(SPDR != 0xFF)
        SPI_Error();
}
//-----------------------------------------------------------------------------------------------------------
void main(void)
{
    AUXR1 |= SPI_Pin_Sel;                       // Select P1.6, P1.7, P1.4, P0.0 as SPI pin(default)
    AUXR1 |= Uart_Port_Sel;                     // Select P10/P11 as UART pin(default)
    InitialUART0_Timer1(9600);                  // 9600 Baud Rate @ 11.0592MHz 
    Show_Version_Number_To_PC();
    printf ("\n*===================================================================");
    printf ("\n*  Name: N79E715 Series SPI Slave(Polling) Sample Code.");
    printf ("\n*===================================================================");
    SPI_Initial();

    printf ("\nSPI Start Receive...\n");


    Slave_Receive_Data();                       // Slave receive data from master
    Slave_tranmit_Data();                       // Slave transmit data to master
    printf ("\nSPI Test OK!\n");
    while(1)
    {
        P14 = 1;
        P21 = 1;
        Delay1ms(500);
        P14 = 0;
        P21 = 0;
        Delay1ms(500);
    }
}
//-----------------------------------------------------------------------------------------------------------
