/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright(c) 2015 Nuvoton Technology Corp. All rights reserved.                                         */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/

//***********************************************************************************************************
//  Nuvoton Technology Corp.
//  E-mail: MicroC-8bit@nuvoton.com
//***********************************************************************************************************
//  Application: Timer0 Function
//  This file set up timer 0 in mode 1 (16 bits timer) with a hardware gate.
//  The 16-bits register consist of all 8 bits of TH0 and all 8 bits of TL0.
//  Timer0 flag will be set when times out. Then ISR will be executed and P1.4 is going to be toggled.
//
//  Output : P1.4 & P2.1 toggle when timer0 interrupt
//           P1.4 & P2.1 toggle period = 2 * 65536 cycles
//***********************************************************************************************************

//------------------------- <<< Use Configuration Wizard in Context Menu >>> --------------------------------
//     <o0.6> UART pin Select
//          <0=> Select P1.0, P1.1 as UART pin(default)
//          <1=> Select P2.6, P2.7 as UART pin(28 pin only)
//-------------------------------- <<< end of configuration section >>> -------------------------------------

#define Uart_Port_Sel   0x00

#include <stdio.h>
#include "N79E715.h"
#include "Typedef.h"
#include "Define.h"
#include "Common.h"
#include "Version.h"
#define  TH0_Value 0x00
#define  TL0_Value 0x00
//-----------------------------------------------------------------------------------------------------------
void main(void)
{
    AUXR1 |= Uart_Port_Sel;         // Select P10/P11 as UART pin(default)
    InitialUART0_Timer1(9600);      // 9600 Baud Rate @ 11.0592MHz
    Show_Version_Number_To_PC();
    printf ("\n*===================================================================");
    printf ("\n*  Name: N79E715 Series Timer0 Sample Code.");
    printf ("\n*===================================================================");
    printf ("\nTimer0 Demo Start.");
    printf ("\nP1.4 toggle when interrupt.");
    printf ("\nP2.1 toggle when interrupt.\n");

    TMOD &= 0xF0;                   // Timer 0 mode 1 with hardware gate
    TMOD |= 0x01;                   // GATE0=0; C/T0#=0; M10=0; M00=1;

    TH0 = TH0_Value;                // Initial values
    TL0 = TL0_Value;
    ET0 = 1;                        // Enable timer0 interrupt
    EA = 1;                         // Enable interrupts
    TR0 = 1;                        // Timer0 run
    while(1);                       // Endless
}
//-----------------------------------------------------------------------------------------------------------
void Timer0_ISR(void) interrupt 1   // Vector @  0x0B
{
    TF0 = 0;                        // Reset timer0 interrupt flag (already done by hardware)
    TH0 = TH0_Value;                // Initial values
    TL0 = TL0_Value;
    P14 = ~ P14;                    // P1.4 toggle when interrupt.
    P21 = ~ P21;                    // P2.1 toggle when interrupt.
}
//-----------------------------------------------------------------------------------------------------------
